writeFasta = function (seq, seqfile="seq.txt", posfile="pos.txt",motiffile="motif.txt") {
	
	if (class(seq)!="seqInfo" & class(seq)!="motif" & class(seq)!="seqData" & class(seq)!="list") {
		warning("argument must be list of sequences or of class seqInfo, motif or seqData")}

	if (class(seq)=="list"){
		seq=new("seqInfo", seqDescrip="", number=length(seq), seqNames=names(seq), 
		lenSum=table(1), prop=matrix(), seqList=seq,objectCall=match.call())
	}

	if (class(seq)=="seqInfo") {
		seqList=seq@seqList
		names(seqList)=paste(">",names(seqList),sep="")
		write.table(unlist(seqList),file=seqfile,quote=FALSE,sep="\n",col.names=FALSE)
	}

	if (class(seq)=="motif") {
		seqList=seq@motifs
		names(seqList)=paste(">",names(seqList),sep="")
		write.table(unlist(seqList),file=motiffile,quote=FALSE,sep="\n",col.names=FALSE)
		d=data.frame("strand"=seq@strand,"pos"=seq@pos)
		write.table(d,file=posfile,quote=FALSE)
	}

	if (class(seq)=="seqData") {	
		seqList=seq@seqInfo@seqList
		names(seqList)=paste(">",names(seqList),sep="")
		write.table(unlist(seqList),file=seqfile,quote=FALSE,sep="\n",col.names=FALSE)
		seqList=seq@motif@motifs
		names(seqList)=paste(">",names(seqList),sep="")
		write.table(unlist(seqList),file=motiffile,quote=FALSE,sep="\n",col.names=FALSE)
		d=data.frame("strand"=seq@motif@strand,"pos"=seq@motif@pos)
		write.table(d,file=posfile,quote=FALSE)
	}
	
}

readFasta <- function(file=NULL) {
	if (is.null(file)) {file=fileBrowser(textToShow="path to sequences (in FASTA format)")}
	all<-readLines(file)
	filelen<-length(all)
	seqlist<-list()
	name_indices<-grep(">",all)
	nseqs<-length(name_indices)

	for (i in 1:nseqs) {
		if (i==nseqs) { end<-filelen }
		else { end<-name_indices[i+1]-1 }

		seq<-paste(all[(name_indices[i]+1):end],sep="",collapse="")
		name<-all[name_indices[i]]
		seqlist[[substring(name,2)]]<-seq
	}
	seqlist
}

writeConstraint = function(constraint, stringOutput=TRUE, outputFile="constraints.txt"){

	if (class(constraint)=="constraintSet") {constraint=new("constraintGroup", description="", numSets=1, constraintSets=list(constraint), objectCall=match.call())}
	if (class(constraint)!="constraintGroup") warning("argument class must be constraintSet or constraintGrp")

	for (i in 1:constraint@numSets) {

		text=rbind(paste("@ ConstraintSet:",i),"", ">IntervalSetup")

	### Interval-specific constraints

		for (j in 1:constraint@constraintSets[[i]]@numIntervals) {
			if (constraint@constraintSets[[i]]@intervals[[j]]@type=="B"){
				text=rbind(text, paste("Length: ",constraint@constraintSets[[i]]@intervals[[j]]@length," bp",sep=""))
			}
			else if (constraint@constraintSets[[i]]@intervals[[j]]@type=="P"){
				text=rbind(text, paste("Length: ",constraint@constraintSets[[i]]@intervals[[j]]@prop,"%",sep=""))
			}
			else if (constraint@constraintSets[[i]]@intervals[[j]]@type=="V"){
				text=rbind(text, "Length: variable")
			}
		}

		text=rbind(text,"")

		for (j in 1:constraint@constraintSets[[i]]@numIntervals) {

			if (dim(constraint@constraintSets[[i]]@intervals[[j]]@boundedCon)[1]!=0) {
				for (k in 1:nrow(constraint@constraintSets[[i]]@intervals[[j]]@boundedCon)) {
					text=rbind(text,">IcBounds",
					paste("Interval: ",j,sep=""),
					paste("Bounds: ",
						constraint@constraintSets[[i]]@intervals[[j]]@boundedCon$Lower[k], " to ",
						constraint@constraintSets[[i]]@intervals[[j]]@boundedCon$Upper[k],"", sep=""),"")
				}
			}

			if (dim(constraint@constraintSets[[i]]@intervals[[j]]@posFreqCon)[1]!=0) {
				for (k in 1:nrow(constraint@constraintSets[[i]]@intervals[[j]]@posFreqCon)) {
					text=rbind(text,">NucFreq",
					paste("Interval: ",j,sep=""),
					paste("Pos: ",constraint@constraintSets[[i]]@intervals[[j]]@posFreqCon$Position[k],sep=""),
					paste("Nuc: ",constraint@constraintSets[[i]]@intervals[[j]]@posFreqCon$Nucleotide[k],sep=""),
					paste("LowerBound: ",constraint@constraintSets[[i]]@intervals[[j]]@posFreqCon$Lower[k],sep=""),
					"")
				}
			}	

			if (dim(constraint@constraintSets[[i]]@intervals[[j]]@shapeCon)[1]!=0) {
				for (k in 1:dim(constraint@constraintSets[[i]]@intervals[[j]]@shapeCon)[1]) {
					text=rbind(text,">IcShape",
					paste("Interval: ",j,sep=""),
					paste("Shape:",constraint@constraintSets[[i]]@intervals[[j]]@shapeCon$Shape[k]),
					paste("LeftBounds: ",constraint@constraintSets[[i]]@intervals[[j]]@shapeCon$StartLower[k]," to ",constraint@constraintSets[[i]]@intervals[[j]]@shapeCon$StartUpper[k],sep=""),
					paste("RightBounds: ",constraint@constraintSets[[i]]@intervals[[j]]@shapeCon$EndLower[k]," to ",constraint@constraintSets[[i]]@intervals[[j]]@shapeCon$EndUpper[k],sep=""),
					paste("ErrorTol:",constraint@constraintSets[[i]]@intervals[[j]]@shapeCon$Error[k]),"")
				}
			}

		}

	### Global constraints		

		if (dim(constraint@constraintSets[[i]]@shapeDiffCon)[1]!=0) {
			for (k in 1:dim(constraint@constraintSets[[i]]@shapeDiffCon)[1]) {
				int1=constraint@constraintSets[[i]]@linCon$Interval1[k]
				int2=constraint@constraintSets[[i]]@linCon$Interval2[k]
				if (int1==int2) {int1=paste(int1,"a",sep="") ; int2=paste(int2,"b",sep="")}
				else {int1=paste(int1,"b",sep="") ; int2=paste(int2,"a",sep="")}
				text=rbind(text,">ParmDiff",
				paste("Parameters: ",int1," - ",int2, sep=""),
				paste("Bounds: ",constraint@constraintSets[[i]]@linCon$Lower[k]," to ",constraint@constraintSets[[i]]@linCon$Upper[k],sep=""),"")
			}
		}

		if (dim(constraint@constraintSets[[i]]@palCon)[1]!=0) {
			for (k in 1:dim(constraint@constraintSets[[i]]@palCon)[1]) {
				text=rbind(text,">Pal",
				paste("Intervals: ",constraint@constraintSets[[i]]@palCon$Interval1[k]," and ",constraint@constraintSets[[i]]@palCon$Interval2[k], sep=""),
				paste("ErrorTol: ",constraint@constraintSets[[i]]@palCon$ErrorBound[k],sep=""),"")
			
			}
		}

		if (dim(constraint@constraintSets[[i]]@subMotifCon)[1]!=0) {
			for (k in 1:dim(constraint@constraintSets[[i]]@subMotifCon)[1]) {
				text=rbind(text,">SubMotif",
				paste("Motif: ",constraint@constraintSets[[i]]@subMotifCon$Submotif[k],sep=""),
				paste("MinFreq: ",constraint@constraintSets[[i]]@subMotifCon$MinFreq[k],sep=""),"")
			}
		}	

	}

	if (stringOutput){
		text=rbind(text,"")
		paste(text[text!=""],collapse="\n")
	}
	else{
		write.table(text,file=outputFile,quote=FALSE,col.names=FALSE,row.names=FALSE,append=TRUE)
		outputFile
	}

}

#FASTA sequence/ sequence list -> sequence matrix 
trans=function(sequenceData=NULL) {
	if (class(sequenceData)!="matrix") {
		if (is.null(sequenceData)) {sequenceData=fileBrowser(textToShow="path to sequences (in FASTA format)")}
		if (class(sequenceData)=="character") {seqlist=readFasta(sequenceData)}
		if (class(sequenceData)=="seqData") {seqlist=sequenceData@list}
		if (class(sequenceData)=="list") {seqlist=sequenceData}
		max=max(unlist(lapply(seqlist,nchar)))
		seqMat=matrix(0,nrow=max,ncol=length(seqlist))
		for (j in 1:length(seqlist)){
			for (i in 1:nchar(seqlist[[j]])){
				seqMat[i,j]=(1:5)[c("A","C","G","T","X")==substring(seqlist[[j]],i,i)]
			}
		}
	}
	else {
		seqMat=sequenceData
	}
	seqMat
}

#seq2counts=function(motifseqs) {
#	splits=sapply(motifseqs,function(x){strsplit(x,split="")[[1]]})
#	counts=apply(splits,1, function(x) {
#		c(sum(x=="A"), sum(x=="C"), sum(x=="G"), sum(x=="T"))
#	})
#	rownames(counts)=c("A","C","G","T")
#	colnames(counts)=1:ncol(counts)
#	counts
#}

motif2pwm=function(motifs) {
	if (class(motifs)=="motif") {motifs=motifs@motifList}
	if (class(motifs)=="seqData") {motifs=motifs@motif@motifList}
	motifseqs=motifs[!is.na(motifs)]
	splits=sapply(unlist(motifseqs),function(x){strsplit(x,split="")[[1]]})
	w=dim(splits)[1]
	pwm=apply(splits,1,function(x){c(sum(x=="A",na.rm=TRUE),sum(x=="C",na.rm=TRUE),sum(x=="G",na.rm=TRUE),sum(x=="T",na.rm=TRUE))})
	rownames(pwm)=c("A","C","G","T")
	colnames(pwm)=1:ncol(pwm)
	pwm/dim(splits)[2]
}

pwm2ic=function(pwm) {
	if (class(pwm)!="matrix") {warning("pwm argument must be of class matrix")}
	npos=ncol(pwm)
	ic=numeric(length=npos)
	for (i in 1:npos) {
		ic[i]=2+sum(sapply(pwm[,i],function(x) { 
			if (x > 0) { x*log2(x) } else { 0 }
		}))
	}	
	ic
}

pwm2cons=function(pwm) {
	if (class(pwm)!="matrix") {warning("pwm argument must be of class matrix")}
	letters = c("A","C","G","T")
	paste(apply(pwm,2,function(x){letters[rev(order(x))[1]]}),collapse="")
}

motif2cons=function(motifs) {
	if (class(motifs)=="motif") {motifs=motifs@motifList}
	if (class(motifs)=="seqData") {motifs=motifs@motif@motifList}
	pwm2cons(motif2pwm(motifs))
}

motif2ic=function(motifs) {
	if (class(motifs)=="motif") {motifs=motifs@motifList}
	if (class(motifs)=="seqData") {motifs=motifs@motif@motifList}
	pwm2ic(motif2pwm(motifs))
}


# Distance
matDist=function(pwm,back.trans.mat,markov) {
	1/(4^markov)*sum(apply(pwm,2,function(x){sum((back.trans.mat[1:4^markov]-rep(1,4^markov)%*%t(x))^2)}))
}

# Flip
flip=function(seq){
	sapply(seq,function(x){
		c("T","G","C","A")[x==c("A","C","G","T")]
	})
}

# Complement
comp=function(seq){
	paste(rev(strsplit(seq,split="")[[1]]),collapse="")
}

#adjust: for ouput class, adds NAs to missing sequences (have lengths match)
#adjust outputs a list of vectors
#adjustchange creates a new motif.set class object with new positions containing the NAs
#result only uses adjust, which takes in objects either of class motif.set or motif

outPos=function(output,numSeq) {
	outPos=rep(as.numeric(NA),numSeq)
	outPos[output@seq]=output@start
	outPos		
}
outStrand=function(output,numSeq) {
	outStrand=rep(as.numeric(NA),numSeq)
	outStrand[output@seq]=output@strand
	outStrand		
}

# Output interpretation

# function to score results for case where we ignore reverse complement
score = function(comode.out, starts){
	trueStarts = starts[comode.out$seq]
	correct = comode.out$start==trueStarts
	sens = sum(comode.out$start==trueStarts)/length(starts)
	spec = sum(comode.out$start==trueStarts)/length(comode.out$start) 
	list(sens=sens,spec=spec)
}

result1motif=function(inputObj,outputObj){

	if (class(outputObj)!="output") {warning("outputObj argument must be of class output")}

	if (class(inputObj)=="seqData") {
		w=inputObj@motif@width
		true=inputObj@motif@pos
		num=inputObj@seqInfo@number
	}
	if (class(inputObj)=="motif") {
		w=inputObj@width
		true=inputObj@pos
		num=length(inputObj@motifs)
	}

	predwid=outputObj@width
	pred=outpos(outputObj,num)

	correctwidth=predwid==w

	nucleotide=apply(cbind(true,pred),1,function(x){
		if (is.na(x[1]) & is.na(x[2])) {
			TP=0
			FN=0
			FP=0
			TN=predwid
		}
		if (!is.na(x[1]) & is.na(x[2])) {
			TP=0
			FN=w
			FP=0
			TN=0
		}
		if (is.na(x[1]) & !is.na(x[2])) {
			TP=0
			FN=0
			FP=predwid
			TN=0
		}
		if (!is.na(x[1]) & !is.na(x[2])) {
			strand1=x[1]:(x[1]+w-1)
			strand2=x[2]:(x[2]+predwid-1)
			TP=length(x[(1:length(strand1))*(apply(sapply(strand2,function(t){strand1==t}),1,sum))])
			FN=w-TP
			FP=predwid-TP
			TN=0
		}
		c(TP,FN,FP,TN)
	})

	nuclevel=apply(nucleotide,1,sum)
	nTP=nuclevel[1]
	nFN=nuclevel[2]
	nFP=nuclevel[3]
	nTN=nuclevel[4]

	sTP=sum(nucleotide[1,]/w >=0.25)
	sFN=sum(nucleotide[1,]/w <0.25 & !is.na(true))
	sFP=sum(nucleotide[1,]/w <0.25 & !is.na(pred))

	nSn=nTP/(nTP+nFN)
	sSn=sTP/(sTP+sFN)
	nPPV=nTP/(nTP+nFP)
	sPPV=sTP/(sTP+sFP)
	nSp=nTN/(nTN+nFP)
	nPC=nTP/(nTP+nFN+nFP)
	nCC=(nTP*nTN-nFN*nFP)/sqrt((nTP+nFN)*(nTN+nFP)*(nTP+nFP)*(nTN+nFN))
	sASP=(sSn+sPPV)/2

	RES=apply(cbind(true,pred),1,function(x){
		if (is.na(x[1]) & is.na(x[2])) {res=-1}
		if (!is.na(x[1]) & is.na(x[2])) {res=-2}
		if (is.na(x[1]) & !is.na(x[2])) {res=-3}
		if (!is.na(x[1]) & !is.na(x[2])) {
			strand1=x[1]:(x[1]+w-1)
			strand2=x[2]:(x[2]+predwid-1)
			res=length(x[(1:length(strand1))*(apply(sapply(strand2,function(t){strand1==t}),1,sum))])
		}
		res
	})

	TPbin=length(RES[RES>=0.25*w])
	FNbin=length(RES[RES==-2])
	FPbin=length(RES[RES==-3 | (RES>=0 & RES<0.25*w)])
	TNbin=length(RES[RES==-1])

	TPstart=sum(!is.na(true) & !is.na(pred) & true==pred)
	TNstart=sum(is.na(true) & is.na(pred))
	FNstart=sum(is.na(pred) & !is.na(true))
	FPstart=sum(!is.na(pred) & (is.na(true) | (!is.na(true) & true!=pred)))

	stat=c(correctwidth,nTP,nTN,nFP,nFN,sTP,sFN,sFP,nSn,sSn,nPPV,sPPV,nSp,nPC,nCC,sASP,TPbin,TNbin,FPbin,FNbin,truepos,trueneg,falsepos,falseneg)
	names(stat)=c("correctwidth","nTP","nTN","nFP","nFN","sTP","sFN","sFP","nSn","sSn","nPPV","sPPV","nSp","nPC","nCC","sASP","TPbin","TNbin","FPbin","FNbin","TPstart","TNstart","FPstart","FNstart")
	stat
}

compare=function(inputObj,outputObj){

	if (class(outputObj)!="output") {warning("outputObj argument must be of class output")}

	if (class(inputObj)=="seqData") {
		w=inputObj@motif@width
		truepos=inputObj@motif@pos
		truestrand=inputObj@motif@strand
		num=inputObj@seqInfo@number
	}
	if (class(inputObj)=="motif") {
		w=inputObj@width
		truepos=inputObj@pos
		truestrand=inputObj@strand
		num=length(inputObj@motifs)
	}

	predwid=outputObj@width
	predpos=outpos(outputObj,num)
	predstrand=outstrand(output,num)

	correctwidth = predwid==w

	TP=FN=FP=TN=0
	sTP=sFN=sFP=0

	for (i in 1:num) {
		if (is.na(truepos[i]) & is.na(predpos[i])) {
			TN=TN+predwid
		}
		if (!is.na(truepos[i]) & is.na(predpos[i])) {
			FN=FN+w
			sFN=sFN+1
		}
		if (is.na(truepos[i]) & !is.na(predpos[i])) {
			FP=FP+predwid
			sFP=sFP+1
		}
		if (!is.na(truepos[i]) & !is.na(predpos[i])) {
			if (truestrand==predstrand) {
				strand1=truepos[i]:(truepos[i]+w-1)
				strand2=predpos[i]:(predpos[i]+predwid-1)
				TP=TP+sum(is.element(strand2,strand1))
				FN=FN+w-TP
				FP=FP+predwid-TP
				sTP=sTP+as.numeric(sum(is.element(strand2,strand1))/w>=0.25)
				sFN=sFN+as.numeric(sum(is.element(strand2,strand1))/w<0.25)
				sFP=sFP+as.numeric(sum(is.element(strand2,strand1))/w<0.25)
			}
			else {
				FP=FP+predwid
			}
		}
	}

	nSn=nTP/(nTP+nFN)
	sSn=sTP/(sTP+sFN)
	nPPV=nTP/(nTP+nFP)
	sPPV=sTP/(sTP+sFP)
	nSp=nTN/(nTN+nFP)
	nPC=nTP/(nTP+nFN+nFP)
	nCC=(nTP*nTN-nFN*nFP)/sqrt((nTP+nFN)*(nTN+nFP)*(nTP+nFP)*(nTN+nFN))
	sASP=(sSn+sPPV)/2

	RES=apply(cbind(truepos,predpos,truestrand,predstrand),1,function(x){
		if (is.na(x[1]) & is.na(x[2])) {res=-1}
		if (!is.na(x[1]) & is.na(x[2])) {res=-2}
		if (is.na(x[1]) & !is.na(x[2])) {res=-3}
		if (!is.na(x[1]) & !is.na(x[2])) {
			strand1=x[1]:(x[1]+w-1)
			strand2=x[2]:(x[2]+predwid-1)
			if (x[3]==x[4]) {
				res=sum(is.element(strand2,strand1))
			}
			else {
				res=0
			}
		}
		res
	})

	TPbin=length(RES[RES>=0.25*w])
	FNbin=length(RES[RES==-2])
	FPbin=length(RES[RES==-3 | (RES>=0 & RES<0.25*w)])
	TNbin=length(RES[RES==-1])

	TPstart=sum(!is.na(truepos) & !is.na(predpos) & truepos==predpos & truestrand==predstrand)
	TNstart=sum(is.na(truepos) & is.na(predpos))
	FNstart=sum(is.na(predpos) & !is.na(truepos))
	FPstart=sum(!is.na(predpos) & (is.na(truepos) | (!is.na(predpos) & truepos!=predpos)) | (!is.na(predpos) & truestrand==predstrand))

	stat=c(correctwidth,nTP,nTN,nFP,nFN,sTP,sFN,sFP,nSn,sSn,nPPV,sPPV,nSp,nPC,nCC,sASP,TPbin,TNbin,FPbin,FNbin,truepos,trueneg,falsepos,falseneg)
	names(stat)=c("correctwidth","nTP","nTN","nFP","nFN","sTP","sFN","sFP","nSn","sSn","nPPV","sPPV","nSp","nPC","nCC","sASP","TPbin","TNbin","FPbin","FNbin","TPstart","TNstart","FPstart","FNstart")
	stat
}

upperTriMat=function(values, byrow=FALSE) {
	l=length(values)
	d=(-1+sqrt(1+8*l))/2
	if (abs(d-round(d))>1e-10) {warning("incorrect values length for upper triangle square matrix")}
	mat=matrix(0,nrow=d,ncol=d)
	if (byrow) {
		for (i in 1:d){
			for (j in 1:(d-i+1)){
				mat[i,j]=values[(i-1)*d+j-(i-1)*(i-2)/2]
			}
		}
	}
	else {
		for (i in 1:d){
			for (j in 1:(d-i+1)){
				mat[i,j]=values[(j-1)*d+i-(j-1)*(j-2)/2]
			}
		}
	}
	mat
}

