# Background

#zero order Markov

backSeqGen0=function(l,p) {
	seqmat=rmultinom(l,1,p)
	c("A","C","G","T")[1:4*seqmat]
}

#first order Markov

backSeqGen1=function(l,transmat){
	oldLetter=rmultinom(1,1,transmat[5,])
	string=c("A","C","G","T")[1:4*oldLetter]
	for (i in 2:l){
		newLetter=rmultinom(1,1,t(oldLetter)%*%transmat[1:4,])
		string=c(string,c("A","C","G","T")[1:4*newLetter])
		oldLetter=newLetter
	}
	string
}

#second order Markov

backSeqGen2=function(l,transmat){
	first=rmultinom(1,1,transmat[21,])
	string=c("A","C","G","T")[1:4*first]
	second=rmultinom(1,1,t(first)%*%transmat[17:20,])
	string=c(string,c("A","C","G","T")[1:4*second])
	last2=first
	last1=second
	transmat=transmat[1:16,]
	for (i in 3:l){
		proba=t(last2)%*%transmat[rep(last1,4)*1:16,]
		newLetter=rmultinom(1,1,proba)
		string=c(string,c("A","C","G","T")[1:4*newLetter])
		last2=last1
		last1=newLetter
	}
	string
}

# convert back matrices for first nucleotides

convert=function(transmat) {
	transmatCopy=transmat
	mkv=round(log(dim(transmat)[1])/log(4))
	if (mkv==log(dim(transmat)[1])/log(4)) {
		for (x in mkv:1) {
			for (line in 1:4^(x-1)) {
				index=seq(from=line, by=4^(x-1),length=4^(mkv-x+1))
				transmatCopy=rbind(transmatCopy,apply(transmat[index,],2,sum)/length(index))	
			}
		}
	}
	transmatCopy
}

# any order markov (assumes transmat already converted)

backSeqGen=function(l, mkv, transmat) {
	last=dim(transmat)[1]
	first=rmultinom(1,1,transmat[last,])
	string=c("A","C","G","T")[1:4*first]
	record=c()
	record=cbind(record,first)
	for (i in 2:mkv) {
		index=1+sum(diag(rev(4^(0:(i-2)))%*%t(0:3)%*%record))
		mat=transmat[(last-(4^i-1)/3+1):(last-(4^(i-1)-1)/3),]
		proba=mat[index,]
		new=rmultinom(1,1,proba)
		record=cbind(record,new)
		string=c(string,c("A","C","G","T")[1:4*new])
	}

	for (j in (mkv+1):l) {
		index=1+sum(diag(rev(4^(0:(mkv-1)))%*%t(0:3)%*%record))
		mat=transmat[1:4^mkv,]
		proba=mat[index,]
		new=rmultinom(1,1,proba)
		string=c(string,c("A","C","G","T")[1:4*new])
		record=cbind(record,new)
		record=record[,-1]
	}
	string
}


#motif generator

motifSeqGen=function(pmat){
	seqmat=apply(pmat,2,function(x){rmultinom(1,1,x)})
	c("A","C","G","T")[1:4*seqmat]
}


#Add One Motif

addOneMotif=function(backgrd, fore.trans.mat,pstrand) {
	w=dim(fore.trans.mat)[2]
	l=length(backgrd)
	start.pos=round(runif(1,1,l-w+1))
	foregrd=motifSeqGen(fore.trans.mat)
	sequence=backgrd
	if (runif(1)<pstrand) {
		sequence[start.pos:(start.pos+w-1)]=rev(flip(foregrd))
		strand="-"
	}
	else {
		sequence[start.pos:(start.pos+w-1)]=foregrd
		strand="+"
	}
	L=list(sequence,strand,start.pos,NA)
	L	
}

#Add Two Motifs

addTwoMotif=function(backgrd, forTransMat1, forTransMat2, pfirst, pstrand) {
	w1=dim(forTransMat1)[2]
	w2=dim(forTransMat2)[2]
	l=length(backgrd)
	if (runif(1)<pfirst) {
		start.pos1=round(runif(1,1,l-w1-w2+1))
		start.pos2=round(runif(1,start.pos1+w1,l-w2+1))
		sequence=backgrd
		foregrd1=motifSeqGen(forTransMat1)
		foregrd2=motifSeqGen(forTransMat2)
		if (runif(1)<pstrand) {
			sequence[start.pos1:(start.pos1+w1-1)]=rev(flip(foregrd1))
			sequence[start.pos2:(start.pos2+w2-1)]=rev(flip(foregrd2))
			strand="-"
		}
		else {
			sequence[start.pos1:(start.pos1+w1-1)]=foregrd1
			sequence[start.pos2:(start.pos2+w2-1)]=foregrd2
			strand="+"
		}
		
	}
	else {
		start.pos2=round(runif(1,1,l-w1-w2+1))
		start.pos1=round(runif(1,start.pos2+w2,l-w1+1))
		sequence=backgrd
		foregrd1=motifSeqGen(forTransMat1)
		foregrd2=motifSeqGen(forTransMat2)
		if (runif(1)<pstrand) {
			sequence[start.pos1:(start.pos1+w1-1)]=foregrd1
			sequence[start.pos2:(start.pos2+w2-1)]=foregrd2
			strand="+"
		}
		else {
			sequence[start.pos1:(start.pos1+w1-1)]=rev(flip(foregrd1))
			sequence[start.pos2:(start.pos2+w2-1)]=rev(flip(foregrd2))
			strand="-"
		}
	}
	L=list(sequence, strand, start.pos1, start.pos2)
	L
}


multiMotifGen=function(l,markov,backTransMat,fore.trans.mat,lambda, pstrand){
	if (markov==0) { backgrd=backSeqGen0(l,convert(backTransMat)) }
	if (markov==1) { backgrd=backSeqGen1(l,convert(backTransMat)) }
	if (markov==2) { backgrd=backSeqGen2(l,convert(backTransMat)) }
	if (markov>2)  { backgrd=backSeqGen(l, markov, convert(backTransMat)) }
	end=0
	dist=round(rexp(1,lambda))
	w=dim(fore.trans.mat)[2]
	sequence=backgrd
	positions=c()
	if (runif(1)<pstrand) {
		strand="-"
		while (end+dist+w<=length(backgrd)) {
			foregrd=motifSeqGen(fore.trans.mat)
			sequence[(end+dist+1):(end+dist+w)]=rev(flip(foregrd))
			positions=c(positions,end+dist+1)
			end=end+dist+w
			dist=round(rexp(1,lambda))
		}
	}
	else {
		strand="+"
		while (end+dist+w<=length(backgrd)) {
			foregrd=motifSeqGen(fore.trans.mat)
			sequence[(end+dist+1):(end+dist+w)]=foregrd
			positions=c(positions,end+dist+1)
			end=end+dist+w
			dist=round(rexp(1,lambda))
		}
	}

	if (is.null(positions)) {strand=NA}
	L=list(sequence,strand,positions)
	L
}



#generates sequence of lenth l with foreground randomly located on 1:l-w+1

seqGen=function(l, markov, backTransMat, proba1, forTransMat1, proba2, forTransMat2,pfirst,pstrand){
	if (markov==0) { backgrd=backSeqGen0(l, backTransMat) }
	if (markov==1) { backgrd=backSeqGen1(l, convert(backTransMat)) }
	if (markov==2) { backgrd=backSeqGen2(l, convert(backTransMat)) }
	if (markov>2)  { backgrd=backSeqGen(l, markov, convert(backTransMat)) }
	u1=runif(1)
	u2=runif(1)
	if (u1>proba1 & u2>proba2) {
		L=list(backgrd,NA,NA,NA)
	}
	else if (u1<=proba1 & u2>proba2) {
		L=addOneMotif(backgrd,forTransMat1,pstrand)
	}
	else if (u1>proba1 & u2<=proba2) {
		L=addOneMotif(backgrd,forTransMat2,pstrand)
		L[[4]]=L[[3]]
		L[[3]]=NA
	}
	else {
		L=addTwoMotif(backgrd, forTransMat1, forTransMat2, pfirst,pstrand)
	}
	L
}

#generates num sequences of length distributed by default according to N(para1,para2)

multiSeqGen=function(num, markov=NULL, backTransMat, proba1, forTransMat1, proba2=0, forTransMat2=NULL, pfirst=0.5, pstrand=0, outType="o", path="", filename="seqFile", dis=rnorm, para1=100, para2=20){
	if (path=="browse") {path=fileBrowser(textToShow="Folder path")}
	if (is.null(markov)) {markov=round(log(nrow(backTransMat))/log(4))}
	if (class(backTransMat)=="data.frame") {backTransMat=as.matrix(backTransMat)}
	if (class(backTransMat)!="matrix") {warning("backTransMat should be of class matrix")}
	if (dim(backTransMat)[2]!=4) {warning("backTransMat should have 4 columns")}
	lengths=round(dis(num,para1,para2),0)
	data=lapply(1:num, function(i) {seqGen(lengths[i], markov, backTransMat, proba1, forTransMat1, proba2, forTransMat2, pfirst, pstrand)} )
	sequences=sapply(data,function(x){paste(unlist(x[1]),collapse="")})
	strand=unlist(sapply(data,function(x){x[2]}))	
	positions1=unlist(sapply(data,function(x){x[3]}))
	positions2=unlist(sapply(data,function(x){x[4]}))
	L=list("sequences"=sequences,"strand"=strand,"positions1"=positions1,"positions2"=positions2)
	name=sapply(1:length(L$sequences),function(x){paste(">seq",x,sep="")})
	names(L$sequences)=name

	object=NULL
	if (outType=="o" | outType=="O") {
		all=paste(sequences,sep="",collapse="")
		seqs=as.list(sequences)
		names(seqs)=gsub(">","",name)
		objectInfo=new("seqInfo",description="Generated Sequences",number=num,seqNames=name,
		lenSum=summary(lengths),prop=as.matrix(table(strsplit(all,split=""))/nchar(all)),seqList=seqs,objectCall=match.call())
		motifs=as.list(rep(NA,num))
		for (i in 1:num) {
			if(!is.na(positions1[i])) {
				motifs[[i]] = substring(seqs[[i]],positions1[i],positions1[i]+dim(forTransMat1)[2]-1)
			}
		}
		names(motifs)=gsub(">","",name)
		objectMotif=new("motif", description="Generated Motifs", pwm=motif2pwm(motifs), width=dim(forTransMat1)[2], pos=positions1, strand=strand, motifList=motifs, objectCall=match.call())
		transmat=backTransMat[1:4^markov,]
		nuc=c("A","C","G","T")
		colnames(transmat)=nuc
		order=markov-1
		kmers=nuc
		while(order > 0) {
			kmers<-apply (expand.grid(kmers,nuc)[,c(2,1)],1, function(x){ paste(x,collapse="")})
			order=order-1
		}
		if(markov > 0) { rownames(transmat)=kmers }
		else { rownames(transmat) = "proba" }
		objectBack=new("backgrd",description="Generated Background",markov=markov, transmat=transmat, objectCall=match.call())
		object=new("seqData",description="Generated Data",seqInfo=objectInfo,motif=objectMotif, backgrd=objectBack,objectCall=match.call())	
	}
	
	if (outType=="t" | outType=="T") {
		write.table(L$sequences,file=paste(path,filename,"_seq.txt",sep=""),quote=FALSE,sep="\n",col.names=FALSE)
		d=data.frame("strand"=L$strand,"pos1"=L$positions1,"pos2"=L$positions2)
		if (proba2!=0) {write.table(d,file=paste(path,filename,"_pos.txt",sep=""),quote=FALSE) }
		else {write.table(d[,-3],file=paste(path,filename,"_pos.txt",sep=""),quote=FALSE)}
#		L
	}

	if (outType=="b" | outType=="B") {
		write.table(L$sequences,file=paste(path,filename,"_seq.txt",sep=""),quote=FALSE,sep="\n",col.names=FALSE)
		d=data.frame("strand"=L$strand,"pos1"=L$positions1,"pos2"=L$positions2)
		if (proba2!=0) {write.table(d,file=paste(path,filename,"_pos.txt",sep=""),quote=FALSE) }
		else {write.table(d[,-3],file=paste(path,filename,"_pos.txt",sep=""),quote=FALSE)}
	
		all=paste(sequences,sep="",collapse="")
		seqs=as.list(sequences)
		names(seqs)=gsub(">","",name)
		objectInfo=new("seqInfo",description="Generated Sequences",number=num,seqNames=name,
		lenSum=summary(lengths),prop=as.matrix(table(strsplit(all,split=""))/nchar(all)),seqList=seqs, objectCall=match.call())
		motifs=as.list(rep(NA,num))
		for (i in 1:num) {
			if(!is.na(positions1[i])) {
				motifs[[i]] = substring(seqs[[i]],positions1[i],positions1[i]+dim(forTransMat1)[2]-1)
			}
		}
		names(motifs)=gsub(">","",name)
		objectMotif=new("motif",description="Generated Motifs",pwm=motif2pwm(motifs), width=dim(forTransMat1)[2], pos=positions1, strand=strand, motifList=motifs, objectCall=match.call())
		transmat=backTransMat[1:4^markov,]
		nuc=c("A","C","G","T")
		colnames(transmat)=nuc
		order=markov-1
		kmers=nuc
		while(order > 0) {
			kmers<-apply (expand.grid(kmers,nuc)[,c(2,1)],1, function(x){ paste(x,collapse="")})
			order=order-1
		}
		if(markov > 0) { rownames(transmat)=kmers }
		else { rownames(transmat) = "proba" }
		objectBack=new("backgrd", description="Generated Background", markov=markov, transmat=transmat, objectCall=match.call())
		object=new("seqData", description="Generated Data", seqInfo=objectInfo, motif=objectMotif, backgrd=objectBack, objectCall=match.call())	
		}
	object
}



multiMotifSeqGen=function( num, markov=NULL, backTransMat, forTransMat, lambda, pstrand=0, outType="o", path="", filename="seqFile", dis=rnorm, para1=100, para2=20){
	if (path=="browse") {path=fileBrowser(textToShow="Folder path")}
	if (is.null(markov)) {markov=round(log(nrow(backTransMat))/log(4))}
	if (class(backTransMat)=="data.frame") {backTransMat=as.matrix(backTransMat)}
	if (class(backTransMat)!="matrix") {warning("backTransMat should be of class matrix")}
	if (dim(backTransMat)[2]!=4) {warning("backTransMat should have 4 columns")}
	lengths=round(dis(num,para1,para2),0)
	data=lapply(1:num, function(i) {multiMotifGen(lengths[i], markov, backTransMat, forTransMat, lambda, pstrand)} )
	sequences=sapply(data,function(x){paste(unlist(x[1]),collapse="")})
	strands=unlist(sapply(data,function(x){x[2]}))
	positions=sapply(data,function(x){x[3]})
	name=sapply(1:length(lengths),function(x){paste(">seq",x,sep="")})
	names(sequences)=name
	names(positions)=gsub(">","",name)
	L=list("sequences"=sequences,"strands"=strands,"positions"=positions)

	object=NULL
	if (outType=="o"|outType=="O") 	{
		all=paste(sequences,sep="",collapse="")
		seqs=as.list(sequences)
		names(seqs)=gsub(">","",name)
		objectInfo=new("seqInfo",description="Generated Sequences",number=num,seqNames=name,
		lenSum=summary(lengths),prop=as.matrix(table(strsplit(all,split=""))/nchar(all)),seqList=seqs, objectCall=match.call())
		multiMotifs=as.list(rep(NA,num))
		for (i in 1:num) {
			if (prod(!is.null(positions[[i]]))==1) {
				multiMotifs[[i]] = sapply(1:length(positions[[i]]),function(z){substring(seqs[[i]],positions[[i]][z],positions[[i]][z]+dim(forTransMat)[2]-1)})
			}
		}
		names(multiMotifs)=gsub(">","",name)
		objectMultiMotif=new("multiMotif", description="Generated Multi Motifs", pwm=motif2pwm(list(unlist(multiMotifs)[!is.na(unlist(multiMotifs))])), width=dim(forTransMat)[2], pos=positions, strand=strands, multiMotifList=multiMotifs, objectCall=match.call())
		transmat=backTransMat[1:4^markov,]
		nuc=c("A","C","G","T")
		colnames(transmat)=nuc
		order=markov-1
		kmers=nuc
		while(order > 0) {
			kmers<-apply (expand.grid(kmers,nuc)[,c(2,1)],1, function(x){ paste(x,collapse="")})
			order=order-1
		}
		if(markov > 0) { rownames(transmat)=kmers }
		else { rownames(transmat) = "proba" }
		objectBack=new("backgrd", description="Generated Background", markov=markov, transmat=transmat, objectCall=match.call())
		object=new("multiSeqData", description="Generated Multi Motif Data", seqInfo=objectInfo, multiMotif=objectMultiMotif, backgrd=objectBack, objectCall=match.call())	
	
	}
	if (outType=="t"|outType=="T") 	{
		write.table(L$sequences,file=paste(path,filename,"_seq.txt",sep=""),quote=FALSE,sep="\n",col.names=FALSE)
		dim2=max(sapply(L$positions,length))
		posmatrix=t(sapply(L$positions,function(x){c(x,rep(0,dim2-length(x)))}))
		mat=cbind(unlist(L$strands),as.data.frame(posmatrix))
		colnames(mat)=c("strand",1:ncol(posmatrix))
		write.table(mat,file=paste(path,filename,"_pos.txt",sep=""),quote=FALSE)
		L
	}
	if (outType=="b"|outType=="B") 	{
		all=paste(sequences,sep="",collapse="")
		seqs=as.list(sequences)
		names(seqs)=gsub(">","",name)
		objectInfo=new("seqInfo",description="Generated Sequences",number=num,seqNames=name,
		lenSum=summary(lengths),prop=as.matrix(table(strsplit(all,split=""))/nchar(all)),seqList=seqs, objectCall=match.call())
		multiMotifs=as.list(rep(NA,num))
		for (i in 1:num) {
			if (prod(!is.null(positions[[i]]))==1) {
				multiMotifs[[i]] = sapply(1:length(positions[[i]]),function(z){substring(seqs[[i]],positions[[i]][z],positions[[i]][z]+dim(forTransMat)[2]-1)})
			}
		}
		names(multiMotifs)=gsub(">","",name)
		objectMultiMotif=new("multiMotif", description="Generated Multi Motifs", pwm=motif2pwm(list(unlist(multiMotifs)[!is.na(unlist(multiMotifs))])), width=dim(forTransMat)[2], pos=positions, strand=strands, multiMotifList=multiMotifs, objectCall=match.call())
		transmat=backTransMat[1:4^markov,]
		nuc=c("A","C","G","T")
		colnames(transmat)=nuc
		order=markov-1
		kmers=nuc
		while(order > 0) {
			kmers<-apply (expand.grid(kmers,nuc)[,c(2,1)],1, function(x){ paste(x,collapse="")})
			order=order-1
		}
		if(markov > 0) { rownames(transmat)=kmers }
		else { rownames(transmat) = "proba" }
		objectBack=new("backgrd", description="Generated Background", markov=markov, transmat=transmat, objectCall=match.call())
		object=new("multiSeqData", description="Generated Multi Motif Data", seqInfo=objectInfo, multiMotif=objectMultiMotif, backgrd=objectBack, objectCall=match.call())	
	
		write.table(L$sequences,file=paste(path,filename,"_seq.txt",sep=""),quote=FALSE,sep="\n",col.names=FALSE)
		dim2=max(sapply(L$positions,length))
		posmatrix=t(sapply(L$positions,function(x){c(x,rep(0,dim2-length(x)))}))
		mat=cbind(unlist(L$strands),as.data.frame(posmatrix))
		colnames(mat)=c("strand",1:ncol(posmatrix))
		write.table(mat,file=paste(path,filename,"_pos.txt",sep=""),quote=FALSE)
		L
	}
	object
}
