setMethod("plot",signature(x="seqInfo"), function(x,y="missing",...){
	lengths=unlist(lapply(x@seqList,nchar))
	hist(lengths,xlab="Length (Basepairs)",ylab="Frequency",main="Length Histogram",col="gray")
})

setMethod("plot",signature(x="motif"), function(x,y="missing",...){
	pwm=x@pwm
	IC=2+apply(pwm,2,function(x) sum(x[x!=0]*log(x[x!=0])))
	plot(1:length(IC),IC,xlab="Position",ylab="Information Content",main="Motif Information Content",typ="l",lty=1,lwd=3,ylim=c(-0.03,2),col="red")
	abline(h=c(0,1,2),lty=2)
	for (i in 1:dim(pwm)[2]) {mtext(c("A","C","G","T")[pwm[,i]==max(pwm[,i])][1],1,at=i,line=-1.1,col="gray10")}
})

setMethod("plot",signature(x="multiMotif"), function(x,y="missing",which=1:2,...){
	par(ask=FALSE)
	if (is.element(1,which)) {
		pwm=x@pwm
		IC=2+apply(pwm,2,function(x) sum(x[x!=0]*log(x[x!=0])))
		plot(1:length(IC),IC,xlab="Position",ylab="Information Content",main="Motif Information Content",typ="l",lty=1,lwd=3,ylim=c(-0.03,2),col="red")
		abline(h=c(0,1,2),lty=2)
		for (i in 1:dim(pwm)[2]) {mtext(c("A","C","G","T")[pwm[,i]==max(pwm[,i])],1,at=i,line=-1.1,col="gray10")}
		par(ask=TRUE)
	}
	if (is.element(1,which)) {
		numMotif=sapply(x@multiMotifList,length)
		hist(numMotif,breaks=0:max(numMotif),xlab="Number of Motifs in sequence",ylab="frequency")
	}
	par(ask=FALSE)
})

setMethod("plot",signature(x="backgrd"), function(x,y="missing",...){
	image(1:4,1:4^(x@markov),t(x@transmat[1:4^(x@markov),]),col=heat.colors(10),axes=FALSE,xlab="Following Nucleotide",ylab="First Nucleotides",main="Transition matrix Image")
	rug(1:5-0.5,side=1,ticksize=-0.02)
	rug(1:(4^(x@markov)+1)-1/2,side=2,ticksize=-0.02)
	mtext(c("A","C","G","T"),at=1:4,side=1)
	par(las=1)
	mtext(rownames(x@transmat)[1:4^(x@markov)],at=1:4^(x@markov),side=2)
	par(las=0)
})

setMethod("plot",signature(x="seqData"),function(x,y="missing",which=1:3,...) {
	par(ask=FALSE)
	if (is.element(1,which)) {plot(x@seqInfo); par(ask=TRUE)}
	if (is.element(2,which)) {plot(x@motif); par(ask=TRUE)}
	if (is.element(3,which)) {plot(x@backgrd); par(ask=TRUE)}
	par(ask=FALSE)	
})

setMethod("plot",signature(x="multiSeqData"),function(x,y="missing",which=1:4,...) {
	par(ask=FALSE)
	if (is.element(1,which)) {plot(x@seqInfo); par(ask=TRUE)}
	if (is.element(2,which) & is.element(3,which)) {plot(x@multiMotif); par(ask=TRUE)}
	if (is.element(2,which) & !is.element(3,which)) {plot(x@multiMotif,which=1); par(ask=TRUE)}
	if (!is.element(2,which) & is.element(3,which)) {plot(x@multiMotif,which=2); par(ask=TRUE)}
	if (is.element(4,which)) {plot(x@backgrd); par(ask=TRUE)}
	par(ask=FALSE)	
})

setMethod("plot",signature(x="output"),function(x,y="missing",...) {

  probs = x@probs
  onRevComp = x@revcomp
  seqs = x@seqs
  seqNames = NULL
  
  maxLen  = nrow(probs)
  numSeqs = ncol(probs)

  ht = (numSeqs/50)*17 + 3;
  wt=18;
  fac = (ht/18)**0.55;

  probs = ifelse(onRevComp,-probs,probs)
  if (is.null(seqNames)) seqNames=1:numSeqs
  
  par(oma=c(0,8,0,0))
  plot(1:maxLen,1:maxLen,ylim=c(0,numSeqs+1),xlim=c(1,maxLen),type="n",axes=F,xlab="",ylab="")
  axis(side=2,at=seq(1,numSeqs,by=1), labels = seqNames,las=2,lty=0,cex.axis=0.75/fac)
  axis(side=1,cex.axis=0.75/fac)
  for (i in 1:numSeqs){
    len = sum(seqs[,i]!=0)
    segments(1:len,rep(numSeqs+1-i,len),1:len,numSeqs+1-i+0.5*probs[,i],col=getColor(seqs[,i]),lwd=2.5)
  }


})

setMethod("plot",signature(x="constraintSet"), function(x,y="missing",varLen=4,propLen=4, plot.IC=TRUE, plot.nucFreq=TRUE,...){

	par(mfrow=c(1,1),mar=c(5, 3, 4, 4.5) + 0.1,las=1)
	int=unlist(lapply(x@intervals,function(y){
		if (y@type=="BasePairs" | y@type=="B" | y@type=="b") {
			y@length
		}
		else if (y@type=="Variable" | y@type=="V" | y@type=="v") {
			varLen
		}
		else if (y@type=="Proportion" | y@type=="P" | y@type=="p") {
			propLen
		}}))	
	props=unlist(lapply(x@intervals,function(y){y@prop}))
	types=unlist(lapply(x@intervals,function(y){
		y@type
	}))
	labels=c()
	for (i in 1:length(types)) {
		if (types[i]=="BasePairs" | types[i]=="B" | types[i]=="b") {labels=c(labels,1:int[i])}
		if (types[i]=="Proportion" | types[i]=="P" | types[i]=="p") {labels=c(labels,paste(props[i],"%",sep=""))}
		if (types[i]=="Variable" | types[i]=="V" | types[i]=="v") {labels=c(labels,"?")}
	}
	splits=c(0,cumsum(int))
	lwbd=min(0.49+0.032*sum(int),1)
	text=ifelse(nrow(x@subMotifCon)==0,"Constraint Plot",paste("Constraint Plot \n Submotif present:", as.character(x@subMotifCon[,"Submotif"])))
	plot(0,xlab="",ylab="",main=text,xlim=c(lwbd,sum(int)+1-lwbd),ylim=c(0.07,1.93),col=0,axes=FALSE)
	axis(side=2,at=c(0,1,2))
	axis(side=4,at=c(0,0.5,1,1.5,2),labels=c("0%","25%","50%","75%","100%"))
	par(las=0)
	mtext(text="Information Content",side=2,line=1.9)
	mtext(text="Proportion",side=4,line=2.7)
	abline(v=splits[-c(1,length(splits))]+0.5,lty=2)
	abline(h=c(0,2))
	rug(1:(sum(int)+1)-0.5,ticksize=-0.02,quiet=TRUE)
	if (length(splits[-c(1,length(splits))])!=0) {rug(splits[-c(1,length(splits))]+1/2,ticksize=-0.12,lty=2)}
	ats=c()
	counting=0
	for (i in 1:length(types)) {
		if (types[i]=="BasePairs" | types[i]=="B" | types[i]=="b") {ats=c(ats,counting+1:int[i]) ; counting=counting+int[i]}
		if (types[i]=="Proportion" | types[i]=="P" | types[i]=="p") {ats=c(ats,counting+(propLen+1)/2) ; counting=counting+propLen}
		if (types[i]=="Variable" | types[i]=="V" | types[i]=="v") {ats=c(ats,counting+(varLen+1)/2) ; counting=counting+varLen}
	}
	mtext(labels , side=1 , at=ats,line=0.5)
	mtext(types, at=(splits[-1]+splits[-length(splits)])/2+1/2,side=1,line=2)
	
	for (i in 1:x@numIntervals){

		if (plot.IC) {

			if(dim(x@intervals[[i]]@boundedCon)[1]!=0) {
				lines(c(splits[i],splits[i+1])+1/2,rep(x@intervals[[i]]@boundedCon$Lower,2),lwd=3,col="red")
				lines(c(splits[i],splits[i+1])+1/2,rep(x@intervals[[i]]@boundedCon$Upper,2),lwd=3,col="red")
			}

			if (dim(x@intervals[[i]]@shapeCon)[1]!=0){
				lines(c(splits[i],splits[i+1])+1/2,c(x@intervals[[i]]@shapeCon$StartLower,x@intervals[[i]]@shapeCon$EndLower),lwd=3,col="red")
				lines(c(splits[i],splits[i+1])+1/2,c(x@intervals[[i]]@shapeCon$StartUpper,x@intervals[[i]]@shapeCon$EndUpper),lwd=3,col="red")
			}

		}

		if (plot.nucFreq){

			if (dim(x@intervals[[i]]@posFreqCon)[1]!=0){
				startlevel=0
				if (sum(x@intervals[[i]]@posFreqCon[,2]=="All")>0){
					dint=x@intervals[[i]]@posFreqCon[x@intervals[[i]]@posFreqCon[,2]=="All",]
					for (k in 1:nrow(dint)) {
						rect(splits[i]+1/2,startlevel,splits[i+1]+1/2,startlevel+2*dint$Lower[k],density=20,angle=45,col="blue")
						l=sum(int)
						legend((splits[i]+splits[i+1])/2+0.4748-0.0659*l,startlevel+dint$Lower[k]+0.1, legend=dint$Nucleotide[k], bty="n")
						startlevel=startlevel+2*dint$Lower[k]
					}
				}
				if (sum(x@intervals[[i]]@posFreqCon[,2]!="All")>0){
					dpos=x@intervals[[i]]@posFreqCon[x@intervals[[i]]@posFreqCon[,2]!="All",]
					for (unique in unique(as.numeric(as.character(dpos$Position)))) {
						lines=which(dpos$Position==unique)
						start=startlevel
						for (k in lines) {
							rect(splits[i]+unique-1/2,start,splits[i]+unique+1/2,start+2*dpos$Lower[k],density=40,angle=-45,col="green")
							c=nchar(as.character(x@intervals[[i]]@posFreqCon$Nucleotide[k]))
							l=sum(int)
							legend(splits[i]+unique-0.042-0.05760*l+0.01996*c-0.008*l*c,start+dpos$Lower[k]+0.06,
								legend=dpos$Nucleotide[k], bty="n")
							start=start+2*dpos$Lower[k]
						}
					}
				}
			}

		}

	}
})

setMethod("plot",signature(x="constraintGroup"), function(x, y="missing", which=NULL, varLen=4, propLen=4, plot.IC=TRUE, plot.nucFreq=TRUE,...) {
	if (is.null(which)) {which=1:x@numSets}
	par(ask=FALSE)
	for (i in which) {
		plot(x@constraintSets[[i]],varLen=varLen,propLen=propLen,plot.IC=plot.IC, plot.nucFreq=plot.nucFreq)
		par(ask=TRUE)
	}
	par(ask=FALSE)
})

setMethod("plot",signature(x="intInfo"), function(x, y="missing", ...) {
	set=new("constraintSet", description="",numIntervals=1, intervals=list(x), linCon=data.frame(), palCon=data.frame(), objectCall=match.call())
	plot(set)
})

setMethod("summary",signature(object="seqInfo"), function(object,...){
	object@lenSum
})

setMethod("summary",signature(object="motif"), function(object,...){
	object@pwm
})

setMethod("summary",signature(object="multiMotif"), function(object,...){
	object@pwm
	summary(sapply(object@multiMotifList,length))
})

setMethod("summary",signature(object="backgrd"), function(object,...){
	object@transmat
})

setMethod("summary",signature(object="seqData"), function(object,...){
	L=list("Length Summary"=object@seqInfo@lenSum, "Position Weight Matrix"=object@motif@pwm, "Transition Matrix"=object@backgrd@transmat)
	L
})

setMethod("summary",signature(object="multiSeqData"), function(object,...){
	L=list("Length Summary"=object@seqInfo@lenSum, "Position Weight Matrix"=object@multiMotif@pwm, 
	"Number of motifs per sequence Summary"=summary(sapply(object@multiMotif@multiMotifList,length)), "Transition Matrix"=object@backgrd@transmat)
	L
})

setMethod("summary",signature(object="output"), function(object,...){
	L=object@pwm
	L
})

