setClass("seqInfo",representation(
	description="character",
	number="numeric", 
	seqNames="character",
	lenSum="table",
	prop="matrix",
	seqList="list",
	objectCall="call"))
seqInfo=function(path2seq=NULL,description="seqInfo Object") {
	if (is.null(path2seq)) {path2seq=fileBrowser(textToShow="Path to sequences text file (FASTA format)")}
	if (class(path2seq)!="character") {warning("path2seq argument must be of class character (path to sequences)")}
	seqs<-readFasta(path2seq)
	nseqs<-length(seqs)
	all=paste(seqs,sep="",collapse="")
	propmat=as.matrix(table(strsplit(all,split=""))/nchar(all))
	new("seqInfo", description=description, number=nseqs, seqNames=names(seqs), 
	lenSum=summary(sapply(seqs,nchar)), prop=propmat, seqList=seqs,objectCall=match.call())
}

setClass("motif", representation(
	description="character",
	pwm="matrix",
	width="numeric",
	pos="numeric",
	strand="character",
	motifList="list",
	objectCall="call"))
motif=function(sequences=NULL, positions=NULL, w, pwm=NULL, description="motif Object"){
	if (class(w)!="numeric") {warning("width argument (w) must be of class numeric")}
	if (is.null(sequences)) {sequences=fileBrowser(textToShow="Path to sequences text file (FASTA format)")}
	if (is.null(positions)) {positions=fileBrowser(textToShow="Path to positions text file")}
	if (class(sequences)=="character") {
          seqs<-readFasta(sequences)
        }
        if (class(sequences)=="list") {
          seqs<-sequences
        }
        if (class(sequences)=="matrix") {
          step1=apply(sequences,2,function(x){paste(x,collapse="")})
          step2=sapply(step1,function(x){gsub("1","A", x)})
          step3=sapply(step2,function(x){gsub("2","C", x)})
          step4=sapply(step3,function(x){gsub("3","G", x)})
          seqs=sapply(step4,function(x){gsub("4","T", x)})
        }
	if (class(positions)=="character") {
          p=read.table(file=positions,skip=1)[,3]
        }
        else {p=positions}
        
	nseqs<-length(seqs)
	motifs<-list()
	for (i in 1:nseqs) {
		if(!is.na(p[i])) {
			motifs[[names(seqs)[i]]] = substring(seqs[[i]],p[i],p[i]+w-1)
		}
	}
	if (is.null(pwm)) {pwm=motif2pwm(motifs)}
	if (class(positions)=="character"){
          strand=as.character(read.table(file=path2pos,skip=1)[,2])
        }
        else {
          strand=rep("?",nseqs)
        }
	new("motif", description=description, pwm=pwm, width=w, pos=p, strand=strand, motifList=motifs, objectCall=match.call())
}

setClass("multiMotif", representation(
	description="character",
	pwm="matrix",
	width="numeric",
	pos="list",
	strand="character",
	multiMotifList="list",
	objectCall="call"))
multiMotif=function(path2seq=NULL, path2pos=NULL, w, pwm=NULL, description="multi motif Object"){
	if (class(w)!="numeric") {warning("width argument (w) must be of class numeric")}
	if (is.null(path2seq)) {path2seq=fileBrowser(textToShow="Path to sequences text file (FASTA format)")}
	if (is.null(path2pos)) {path2pos=fileBrowser(textToShow="Path to positions text file")}
	if (class(path2seq)!="character") {warning("path2seq argument must be of class character (path to sequences)")}
	if (class(path2pos)!="character") {warning("path2pos argument must be of class character (path to positions)")}
	p=read.table(file=path2pos)
	pos=lapply(1:dim(p)[1],function(row){
		if (is.na(p[row,"strand"])) {out=NA}
		if (!is.na(p[row,"strand"])) {
			out=as.numeric(p[row,2:dim(p)[2]])
			out=out[out!=0]
		}
		out
	})
	seqs = readFasta(path2seq)
	names(pos)=names(seqs)
	nseqs = length(seqs)
	motifs = list()
	for (i in 1:nseqs) {
		if (!is.na(p[i,"strand"])) {
			motifs[[i]] = sapply(1:length(pos[[i]]),function(x) {substring(seqs[[i]],pos[[i]][x],pos[[i]][x]+w-1)})
		}
	}
	names(motifs) = names(seqs) 
	if (is.null(pwm)) {pwm=motif2pwm(list(unlist(motifs)))}	
	strand=as.character(p[,"strand"])
	new("multiMotif", description=description, pwm=pwm, width=w, pos=pos, strand=strand, multiMotifList=motifs, objectCall=match.call())
}

setClass("backgrd",representation(
	description="character",
	markov="numeric", 
	transmat="matrix",
	objectCall="call"))
backgrd=function(transmat=NULL, markov=0, description="background Object"){
	if (is.null(transmat)) {transmat=fileBrowser(textToShow="Path to transition matrix text file")}
	if (class(transmat)!="character" & class(transmat)!="matrix") {warning("argument must be of class character (path to transition matrix) or matrix")}
	if (class(transmat)=="character") {
		transmat=as.matrix(read.table(file=transmat)[-1])
	}
	if (class(transmat)=="matrix") {
		if (markov!=0 & markov!=round(log(nrow(transmat))/log(4))) {print(paste("Markov Order incompatible with transition Matrix, assuming new order:",log(nrow(transmat))/log(4)))}
		markov=round(log(nrow(transmat))/log(4))
		nuc=c("A","C","G","T")
		colnames(transmat)=nuc
		order=markov-1
		kmers=nuc
		while(order > 0) {
			kmers<-apply (expand.grid(kmers,nuc)[,c(2,1)],1, function(x){ paste(x,collapse="")})
			order=order-1
		}
		if(markov > 0) { rownames(transmat)=kmers }
		else { rownames(transmat) = "proba" }
	}
	
	new("backgrd", description=description, markov=markov, transmat=transmat, objectCall=match.call())
}

setClass("seqData", representation(
	description="character",
	seqInfo="seqInfo",
	motif="motif", 
	backgrd="backgrd",
	objectCall="call"))
seqData=function(path2seq=NULL,path2pos=NULL,w,transmat=NULL,pwm=NULL,description="seqData Object") {
	if (class(w)!="numeric") {warning("width argument (w) must be of class numeric")}
	if (is.null(path2seq)) {path2seq=fileBrowser(textToShow="Path to sequences text file (FASTA format)")}
	if (is.null(path2pos)) {path2pos=fileBrowser(textToShow="Path to positions text file")}
	if (is.null(transmat)) {transmat=fileBrowser(textToShow="Path to transition matrix text file")}
	if (class(path2seq)!="character") {warning("path2seq argument must be of class character (path to sequences)")}
	if (class(path2pos)!="character") {warning("path2pos argument must be of class character (path to positions)")}
	seqInfo=seqInfo(path2seq)
	motif=motif(path2seq,path2pos,w,pwm)
	backgrd=backgrd(transmat)
	new("seqData", description=description, seqInfo=seqInfo, motif=motif, backgrd=backgrd, objectCall=match.call())
}

setClass("multiSeqData", representation(
	description="character",
	seqInfo="seqInfo",
	multiMotif="multiMotif", 
	backgrd="backgrd",
	objectCall="call"))
multiSeqData=function(path2seq=NULL,path2pos=NULL,w,transmat=NULL,pwm=NULL,description="multi motif Data Object") {
	if (class(w)!="numeric") {warning("width argument (w) must be of class numeric")}
	if (is.null(path2seq)) {path2seq=fileBrowser(textToShow="Path to sequences text file (FASTA format)")}
	if (is.null(path2pos)) {path2pos=fileBrowser(textToShow="Path to positions text file")}
	if (is.null(transmat)) {transmat=fileBrowser(textToShow="Path to transition matrix text file")}
	if (class(path2seq)!="character") {warning("path2seq argument must be of class character (path to sequences)")}
	if (class(path2pos)!="character") {warning("path2pos argument must be of class character (path to positions)")}
	seqInfo=seqInfo(path2seq)
	multiMotif=multiMotif(path2seq,path2pos,w,pwm)
	backgrd=backgrd(transmat)
	new("multiSeqData", description=description, seqInfo=seqInfo, multiMotif=multiMotif, backgrd=backgrd, objectCall=match.call())
}

setClass("output", representation(
	width="numeric",
	model="character",
	conset="numeric",
	pwm="matrix",
	consensus="character",
	ic="numeric",
	eval="numeric",
	numSites="numeric",
	alignSeqs="numeric",
	alignStarts="numeric",
	probs="matrix",
	revcomp="matrix",
	sites="matrix",
	order="numeric",
	klDivs="numeric",
	wCrits="numeric",
	modCrits="numeric",
	conCrits="numeric",
	transMats="list",
	seqs="matrix",
	objectCall="call"))
output=function(width, model, conset, pwm, consensus, ic, eval, numSites, alignSeqs, alignStarts, probs, revcomp, sites, order, klDivs, wCrits, modCrits, conCrits, transMats, seqs) {
	new("output", width=width, model=model, conset=conset, pwm=pwm, consensus=consensus, ic=ic, eval=eval, numSites=numSites, alignSeqs=alignSeqs, alignStarts=alignStarts, probs=probs, revcomp=revcomp, sites=sites, order=order, klDivs=klDivs, wCrits=wCrits, modCrits=modCrits, conCrits=conCrits, transMats=transMats, seqs=seqs, objectCall=match.call())
}

setClass("constraintGroup",representation(
	description="character",
	numSets="numeric",
	constraintSets="list",
	objectCall="call"))
setClass("constraintSet", representation(
	description="character",
	numIntervals="numeric", 
	intervals="list",
	shapeDiffCon="data.frame",
	subMotifCon="data.frame",
	palCon="data.frame",
	objectCall="call"))
setClass("intInfo", representation(
	constraintID="numeric",
	intervalID="numeric", 
	type="character",
	length="numeric",
	prop="numeric",
	boundedCon="data.frame",
	posFreqCon="data.frame",
	shapeCon="data.frame",
	objectCall="call"))


constraintGroup = function(path2set=NULL, description="Constraint Group"){
	if (is.null(path2set)) {path2set=fileBrowser(textToShow="Path to constraint set(s)")}
	if (class(path2set)!="character") {warning("path2set argument must be of class character (path to constraint set(s))")}
  	rl = readLines(path2set)
  	l = length(rl)
	collapse = function(x){gsub(" ","",x)}
	catch = function(string,data,element){
     		bound<-unlist(lapply(strsplit(data[grep(paste(string,":",sep=""),data)],split=" "),function(x){collapse(x[element])}))
   		bound	
	}
	spacecatch = function(string,element){
     		strsplit(string," ")[[1]][element]	
	}
	anycatch=function(string, afterwhat){
		strsplit(string,afterwhat)[[1]][2]
	}
	between=function(number,sorted){
		sum(sorted<=number)
	}
	conStart = grep("@ ConstraintSet",rl)
	intsetup = grep(">IntervalSetup",rl)
	numCons=length(set1line)
	conBounds=c(set1line,l+1)

	constraintSets=list()

	for (i in 1:length(conStart)){

	##INTSETUP

		first=intsetup[1]+1
		type=c()
		length=c()
		prop=c()
		while(substring(rl[first],1,7)=="Length:"){
			if (anycatch(rl[first], "Length: ")=="variable"){
				type=c(type,"V")
				length=c(length,NA)
				prop=c(prop,NA)
			}
			else if (length(grep("%",anycatch(rl[first], "Length: ")))!=0){
				type=c(type,"P")
				length=c(length,as.numeric(substring(anycatch(rl[first], "Length: "),1,nchar(anycatch(rl[first], "Length: "))-1)))
			}
			else {
				type=c(type,"B")
				length=c(length,as.numeric(spacecatch(rl[first],2)))
			}
			first=first+1
		}

		constraintSet=makeConSet(length(type),type,length,descrip="Constraint Set")

		### Submotif

		lines=grep(">SubMotif",rl)
		lines=lines[lines>=conBounds[i] & lines<=conBounds[i+1]]
		if (length(lines)>0){
			for (k in 1:length(lines)){
				SMC=makeSubMotifCon(anycatch(rl[lines[k]+1],"Motif: "), as.numeric(anycatch(rl[lines[k]+2],"MinFreq: ")))
				constraintSet=addCon(constraintSet,list(SMC),1)
			}
		}

		### Palindrome

		lines=grep(">Palindrome",rl)
		lines=lines[lines>=conBounds[i] & lines<=conBounds[i+1]]
		if (length(lines)>0){
			for (k in 1:length(lines)){
				PC=makePalCon(as.numeric(spacecatch(rl[lines[k]+1],2)), as.numeric(spacecatch(rl[lines[k]+1],4)), as.numeric(spacecatch(rl[lines[k]+2],2)))
				constraintSet=addCon(constraintSet,list(PC),1)
			}
		}

		### Shape Diff

		lines=grep(">ParmDiff",rl)
		lines=lines[lines>=conBounds[i] & lines<=conBounds[i+1]]
		if (length(lines)>0){
			for (k in 1:length(lines)){
				SDC=makeShapeDiffCon(as.numeric(substring(1,1,spacecatch(rl[lines[k]+1],2))), as.numeric(substring(1,1,spacecatch(rl[lines[k]+1],4))),
					as.numeric(spacecatch(rl[lines[k]+2],2)), as.numeric(spacecatch(rl[lines[k]+2],2)))
				constraintSet=addCon(constraintSet,list(SDC),1)
			}
		}

		### IcBounds

		lines=grep(">IcBounds",rl)
		lines=lines[lines>=conBounds[i] & lines<=conBounds[i+1]]
		if (length(lines)>0){
			for (k in 1:length(lines)){
				ICBC=makeBoundCon(as.numeric(spacecatch(rl[lines[k]+2],2)), as.numeric(spacecatch(rl[lines[k]+2],4)))
				constraintSet=addCon(constraintSet,list(ICBC),as.numeric(spacecatch(rl[lines[k]+1],2)))
			}
		}

		### IcShape

		lines=grep(">IcShape",rl)
		lines=lines[lines>=conBounds[i] & lines<=conBounds[i+1]]
		if (length(lines)>0){
			for (k in 1:length(lines)){
				ICSC=makeShapeCon(anycatch(rl[lines[k]+2],"Shape: "), as.numeric(spacecatch(rl[lines[k]+3],2)), as.numeric(spacecatch(rl[lines[k]+3],4)),
					as.numeric(spacecatch(rl[lines[k]+4],2)), as.numeric(spacecatch(rl[lines[k]+4],4)), as.numeric(spacecatch(rl[lines[k]+5],2)))
				constraintSet=addCon(constraintSet,list(ICSC),as.numeric(spacecatch(rl[lines[k]+1],2)))
			}
		}

		### NucFreq

		lines=grep(">NucFreq",rl)
		lines=lines[lines>=conBounds[i] & lines<=conBounds[i+1]]
		if (length(lines)>0){
			for (k in 1:length(lines)){
				NFC=makePosFreqCon(spacecatch(rl[lines[k]+2],2), spacecatch(rl[lines[k]+3],2), as.numeric(spacecatch(rl[lines[k]+4],2)))
				constraintSet=addCon(constraintSet,list(NFC),as.numeric(spacecatch(rl[lines[k]+1],2)))
			}
		}

		constraintSets[[i]]=constraintSet

	}

	makeConGrp(constraintSets, descrip="Constraint Group read in from text file")

}

makeConSet=function(numInt,type,length,descrip="Constraint Set") {
	if (length(type)!=numInt | length(length)!=numInt ) {warning("arguments length and type must have same length equal to numInt")}
	intervals=lapply(1:numInt,function(x){
		if (type[x]=="B") {
			len=as.numeric(length[x])
			prop=as.numeric(NA)
		}
		if (type[x]=="V") {
			len=as.numeric(NA)
			prop=as.numeric(NA)
		}
		if (type[x]=="P") {
			len=as.numeric(NA)
			prop=as.numeric(length[x])
		}
		new("intInfo", constraintID=1, intervalID=x, type=type[x],
		length=len, prop=prop, boundedCon=data.frame(), posFreqCon=data.frame(),
		shapeCon=data.frame(), objectCall=match.call())
	})
	set=new("constraintSet", description=descrip, numIntervals=numInt, intervals=intervals, shapeDiffCon=data.frame(), subMotifCon=data.frame(), palCon=data.frame(), objectCall=match.call())
	set
}

makeConGrp=function(conSets, descrip="Constraint Group"){
	if (length(conSets)>1 & class(conSets)!="list"){warning("conSets argument must be a list of constraint sets")}
	if (length(conSets)==1 & class(conSets)!="list" & class(conSets)!="constraintSet") {warning("conSets argument must be of class list or constraintSet")}
	if (length(conSets)==1 & class(conSets)!="list") {conSets=list(conSets)}
	if (length(conSets)>1) {
		for (j in 1:length(conSets)) {
			for (i in 1:conSets[[j]]@numIntervals) {
				conSets[[j]]@intervals[[i]]@constraintID=j
			}
			if (dim(conSets[[j]]@shapeDiffCon)[1]>0) {
				conSets[[j]]@shapeDiffCon$Constraint = rep(j,dim(conSets[[j]]@shapeDiffCon)[1])
			}
			if (dim(conSets[[j]]@subMotifCon)[1]>0) {
				conSets[[j]]@subMotifCon$Constraint = rep(j,dim(conSets[[j]]@subMotifCon)[1])
			}
			if (dim(conSets[[j]]@palCon)[1]>0) {
				conSets[[j]]@palCon$Constraint = rep(j,dim(conSets[[j]]@palCon)[1])
			}
		}
	}
	new("constraintGroup", description=descrip, numSets=length(conSets), constraintSets=conSets, objectCall=match.call())
}

setClass("boundCon",representation(
	lower="numeric",
	upper="numeric"))

makeBoundCon=function(lower,upper) {
	new("boundCon",lower=lower, upper=upper)	
}

setClass("posFreqCon",representation(
	pos="character",
	nuc="character",
	lower="numeric"))

makePosFreqCon=function(pos, nuc, lower) {
	new("posFreqCon",pos=pos, nuc=nuc, lower=lower)	
}

setClass("shapeCon",representation(
	shape="character",
	sLower="numeric",
	sUpper="numeric",
	eLower="numeric",
	eUpper="numeric",
	error="numeric"))

makeShapeCon=function(shape, sLower, sUpper, eLower, eUpper, error) {
	new("shapeCon", shape=shape, sLower=sLower, sUpper=sUpper, eLower=eLower, eUpper=eUpper, error=error)	
}

setClass("shapeDiffCon",representation(
	int1="character",
	int2="character",
	lower="numeric",
	upper="numeric"))

makeShapeDiffCon=function(int1, int2, lower, upper) {
	if (nchar(int1)==2 & nchar(int2)==2){
		if (is.na(as.numeric(substring(int1,1,1))) | is.na(as.numeric(substring(int2,1,1))) | !is.element(substring(int1,2,2),c("a","b")) | !is.element(substring(int2,2,2),c("a","b"))) {
			warning("Interval inputs for shape difference constraint must be a concatenation of the interval number and the extremity of that interval ('a' for beginning and 'b' for end)")
		}
	}
	else {
		warning("Interval inputs for shape difference constraint must be a concatenation of the interval number and the extremity of that interval ('a' for beginning and 'b' for end)")
	}
	new("shapeDiffCon", int1=int1, int2=int2, lower=lower, upper=upper)	
}

setClass("subMotifCon",representation(
	submotif="character",
	minfreq="numeric"))

makeSubMotifCon=function(submotif, minfreq) {
	new("subMotifCon", submotif=submotif, minfreq=minfreq)	
}

setClass("palCon",representation(
	int1="numeric",
	int2="numeric",
	errBnd="numeric"))

makePalCon=function(int1, int2, errBnd) {
	new("palCon", int1=int1, int2=int2, errBnd=errBnd)	
}

addCon=function(conSet, constraint, int=1 ) {

	if (class(int) != "list") {int=as.list(int)}
	if (class(constraint) != "list") {warning("constraint argument must be a list")}

	if (length(int) != length(constraint)) {warning("arguments int and constraint must have same length")}
	
		for (i in 1:length(int)) {
			if (class(constraint[[i]])=="boundCon") {
				if (dim(conSet@intervals[[int[[i]]]]@boundedCon)[1]==0) {
					conSet@intervals[[int[[i]]]]@boundedCon=data.frame("Interval"=int[[i]],"Lower"=constraint[[i]]@lower,"Upper"=constraint[[i]]@upper)
				}
				else {
					conSet@intervals[[int[[i]]]]@boundedCon=as.data.frame(t(cbind(t(conSet@intervals[[int[[i]]]]@boundedCon),c(int[[i]], constraint[[i]]@lower, constraint[[i]]@upper))))
					rownames(conSet@intervals[[int[[i]]]]@boundedCon)=1:dim(conSet@intervals[[int[[i]]]]@boundedCon)[1]
				}
			}
			if (class(constraint[[i]])=="intFreqCon") {
				if (dim(conSet@intervals[[int[[i]]]]@intFreqCon)[1]==0) {
					conSet@intervals[[int[[i]]]]@intFreqCon=data.frame("Interval"=int[[i]],"Nucleotide"=constraint[[i]]@nuc,"Lower"=constraint[[i]]@lower)
				}
				else {
					conSet@intervals[[int[[i]]]]@intFreqCon=as.data.frame(t(cbind(t(conSet@intervals[[int[[i]]]]@intFreqCon),c(int[[i]], constraint[[i]]@nuc, constraint[[i]]@lower))))
					rownames(conSet@intervals[[int[[i]]]]@intFreqCon)=1:dim(conSet@intervals[[int[[i]]]]@intFreqCon)[1]
					conSet@intervals[[int[[i]]]]@intFreqCon[,1]=as.numeric(conSet@intervals[[int[[i]]]]@intFreqCon[,1])
					conSet@intervals[[int[[i]]]]@intFreqCon[,2]=as.factor(conSet@intervals[[int[[i]]]]@intFreqCon[,2])
					conSet@intervals[[int[[i]]]]@intFreqCon[,3]=as.numeric(conSet@intervals[[int[[i]]]]@intFreqCon[,3])
				}
			}
			if (class(constraint[[i]])=="posFreqCon") {
				if (dim(conSet@intervals[[int[[i]]]]@posFreqCon)[1]==0) {
					conSet@intervals[[int[[i]]]]@posFreqCon=data.frame("Interval"=int[[i]],"Position"=constraint[[i]]@pos, "Nucleotide"=constraint[[i]]@nuc,"Lower"=constraint[[i]]@lower)
				}
				else {
					conSet@intervals[[int[[i]]]]@posFreqCon=as.data.frame(t(cbind(t(conSet@intervals[[int[[i]]]]@posFreqCon),c(int[[i]], constraint[[i]]@pos, constraint[[i]]@nuc, constraint[[i]]@lower))))
					rownames(conSet@intervals[[int[[i]]]]@posFreqCon)=1:dim(conSet@intervals[[int[[i]]]]@posFreqCon)[1]
					conSet@intervals[[int[[i]]]]@posFreqCon[,1]=as.numeric(conSet@intervals[[int[[i]]]]@posFreqCon[,1])
					conSet@intervals[[int[[i]]]]@posFreqCon[,2]=as.factor(conSet@intervals[[int[[i]]]]@posFreqCon[,2])
					conSet@intervals[[int[[i]]]]@posFreqCon[,3]=as.factor(conSet@intervals[[int[[i]]]]@posFreqCon[,3])
					conSet@intervals[[int[[i]]]]@posFreqCon[,4]=as.numeric(as.character(conSet@intervals[[int[[i]]]]@posFreqCon[,4]))
				}
			}
			if (class(constraint[[i]])=="shapeCon") {
				if (dim(conSet@intervals[[int[[i]]]]@shapeCon)[1]==0) {
					conSet@intervals[[int[[i]]]]@shapeCon=data.frame("Interval"=int[[i]],"Shape"=constraint[[i]]@shape,"StartLower"=constraint[[i]]@sLower,"StartUpper"=constraint[[i]]@sUpper,"EndLower"=constraint[[i]]@eLower,"EndUpper"=constraint[[i]]@eUpper,"Error"=constraint[[i]]@error)
				}
				else {
					conSet@intervals[[int[[i]]]]@shapeCon=as.data.frame(t(cbind(t(conSet@intervals[[int[[i]]]]@shapeCon),c(int[[i]], constraint[[i]]@shape, constraint[[i]]@sLower, constraint[[i]]@sUpper, constraint[[i]]@eLower, constraint[[i]]@eUpper, constraint[[i]]@error))))
					rownames(conSet@intervals[[int[[i]]]]@posFreqCon)=1:dim(conSet@intervals[[int[[i]]]]@shapeCon)[1]
					conSet@intervals[[int[[i]]]]@shapeCon[,1]=as.numeric(conSet@intervals[[int[[i]]]]@shapeCon[,1])
					conSet@intervals[[int[[i]]]]@shapeCon[,2]=as.numeric(conSet@intervals[[int[[i]]]]@shapeCon[,2])
					conSet@intervals[[int[[i]]]]@shapeCon[,3]=as.factor(conSet@intervals[[int[[i]]]]@shapeCon[,3])
					conSet@intervals[[int[[i]]]]@shapeCon[,4]=as.numeric(conSet@intervals[[int[[i]]]]@shapeCon[,4])
				}
			}

			if (class(constraint[[i]])=="subMotifCon") {
				if (dim(conSet@subMotifCon)[1]==0) {
					conSet@subMotifCon=data.frame("Constraint"=1, "Submotif"=constraint[[i]]@submotif, "MinFreq"=constraint[[i]]@minfreq)
				}
				else {
					conSet@subMotifCon=rbind(conSet@subMotifCon,c(1, constraint[[i]]@submotif, constraint[[i]]@minfreq))
					rownames(conSet@subMotifCon)=1:dim(conSet@subMotifCon)[1]
					conSet@subMotifCon[,1]=as.numeric(conSet@subMotifCon[,1])
					conSet@subMotifCon[,2]=as.character(conSet@subMotifCon[,2])
					conSet@subMotifCon[,3]=as.numeric(conSet@subMotifCon[,3])
				}
			}

			if (class(constraint[[i]])=="palCon") {
				if (dim(conSet@palCon)[1]==0) {
					conSet@palCon=data.frame("Constraint"=1, "Interval1"=constraint[[i]]@int1,"Interval2"=constraint[[i]]@int2, "ErrorBound"=constraint[[i]]@errBnd)
				}
				else {
					conSet@palCon=rbind(conSet@palCon,c(1, constraint[[i]]@submotif, constraint[[i]]@int2, constraint[[i]]@errBnd))
					rownames(conSet@palCon)=1:dim(conSet@palCon)[1]
					conSet@palCon[,1]=as.numeric(conSet@palCon[,1])
					conSet@palCon[,2]=as.numeric(conSet@palCon[,2])
					conSet@palCon[,3]=as.numeric(conSet@palCon[,3])
					conSet@palCon[,4]=as.numeric(conSet@palCon[,4])
				}
			}

		}

	conSet
	
}

