\name{cosmo}
\alias{cosmo}
\title{ Constrained motif detection main function}
\description{
  This function is at the heart of the package and performs the constrained motif detection, given a series of sequences and the constraints imposed.  
}
\usage{
  cosmo(seqs = NULL, minW = 6, maxW = 15, revComp = TRUE,
                 minSites = NULL, maxSites = NULL, oops = FALSE, zoops
                 = TRUE, tcm = FALSE, approx = "over", cutFac = 2,
                 wCrit = "eval", wFold = 5, wTrunc = 100, modCrit =
                 "eval", modFold = 5, modTrunc = 100, conCrit =
                 "pwmCV", conFold = 2, conTrunc = 100, intCrit =
                 "eval", intFold = 5, intTrunc = 100, constraints =
                 NULL, starts = 5, lstarts = FALSE, backSeqs = NULL,
                 backFold = 5, transMat = NULL, order = NULL)
}
\arguments{
  \item{seqs}{ this argument contains the sequences to be analyzed. Its class can be of many different types: \code{list}, \code{seqInfo}, 
\code{seqData} or \code{character}. If the argument is left to the default \code{NULL}, a browser enabling to select the text files containing
the sequences in FASTA format appears (requires the \code{tkWidgets} library) }
  \item{minW}{ \code{numeric} indicating the minimum motif width to consider}
  \item{maxW}{ \code{numeric} indicating the maximum motif width to consider}
  \item{revComp}{ \code{logical} indicating whether the complement
    strand should also be considered in the detection}
  \item{minSites}{}
  \item{maxSites}{}
  \item{oops}{ \code{logical} indicating whether a model with Only One motif Per Sequence should be considered}
  \item{zoops}{\code{logical} indicating whether a model with Zero Or One motif Per Sequence should be considered}
  \item{tcm}{\code{logical} indicating whether a model with Zero Or One Per Sequence should be considered}
  \item{approx}{}
  \item{cutFac}{}
  \item{wCrit}{}
  \item{wFold}{}
  \item{wTrunc}{}
  \item{modCrit}{}
  \item{modFold}{}
  \item{modTrunc}{}
  \item{conCrit}{}
  \item{conFold}{}
  \item{conTrunc}{}
  \item{intCrit}{}
  \item{intFold}{}
  \item{intTrunc}{}
  \item{constraints}{ these are the constraints imposed on the motif. Its class can either be \code{constraintGroup}, \code{constraintSet}, \code{intInfo} or 
    \code{character} in which case it is the path to a constraint set or
    group in text format.}
  \item{starts}{}
  \item{lstarts}{}
  \item{backSeqs}{}
  \item{backFold}{}
  \item{transMat}{}
  \item{order}{} 
}
\details{
  The sequence argument can be of many different types: \code{list}, \code{seqInfo}, \code{seqData} or \code{character}. In the latter case, the string is
assumed to be the path to the text file containing the sequences in FASTA format. If the argument is left to the default \code{NULL}, a browser enabling to
select the text files containing the sequences in FASTA format appears (this requires the \code{tkWidgets} library) 

  The constraint argument can also be of many different types: \code{constraintSet}, \code{constraintGroup} or \code{character}. In the latter case, the string
is assumed to be the path to the text file containing the constraints in COSMO format. (~web reference here~). If the argument is left to the default \code{NULL},
a widget will pop-up warning the user that no constraints have been given, and asking whether the user wishes to create one. If the user declines, an 
unconstrained motif detection is performed. If the user accepts, a series of easy-to-follow pop-ups will appear allowing the user to create a constraint set from
scratch or adapt pre-defined constraint sets. The constraint set will be saved and outputed at the end of the call to the cosmo function, allowing it to be 
available for future use.
}
\value{
  An object of class \code{output}, returning all the results of the motif detection analysis.
}
\references{ \url{http://cosmoweb.berkeley.edu/} }
\author{ Fabian Gallusser, \email{fgallusser@berkeley.edu}, Oliver Bembom, \email{bembom@berkeley.edu}}
\note{ ~~further notes~~ }

\seealso{ \code{\link{output}} }
\examples{
data(seqs)
res = cosmo(seqs=seqs,minW=8,maxW=8)
}
\keyword{misc}
