\name{motif-class}
\docType{class}
\alias{motif-class}
\alias{motif}
\alias{plot,motif-method}
\alias{summary,motif-method}

\title{Class "motif" ~~~ }
\description{  An object of class \code{"motif"} summarizes all avaialble data on the motifs in sequences.
 This class is almost only used for generated data, when exact motif sequences and positions are known. It is assumed that the motif is present at most once in a given sequence. }
\section{Objects from the Class}{
Objects can be created by calls of the form 
\code{  new('motif',}\cr
\code{    description     = ...., # Object of class \code{"character"}}\cr
\code{    pwm             = ...., # Object of class \code{"matrix"}}\cr
\code{    width           = ...., # Object of class \code{"numeric"}}\cr
\code{    pos             = ...., # Object of class \code{"numeric"}}\cr
\code{    strand          = ...., # Object of class \code{"character"}}\cr
\code{    motifList       = ...., # Object of class \code{"list"}}\cr
\code{    objectCall      = ...., # Object of class \code{"call"}}\cr
\code{  )}
or by:
\code{  motif(}\cr
\code{    sequences     = NULL            # Object of class \code{"character"}}\cr
\code{    positions     = NULL            # Object of class \code{"character"}}\cr
\code{    w             = ....,           # Object of class \code{"numeric"}}\cr
\code{    pwm           = NULL            # Object of class \code{"matrix"}}\cr
\code{    description   = "motif Object"  # Object of class \code{"character"}}\cr
\code{  )}
}
\section{Slots}{
  \describe{
    \item{\code{description}:}{Object of class \code{"character"} general description of the motifs }
    \item{\code{pwm}:}{Object of class \code{"matrix"} position weight matrix for the motif }
    \item{\code{width}:}{Object of class \code{"numeric"} width of the motif }
    \item{\code{pos}:}{Object of class \code{"numeric"} numeric vector of the starting positions of the motifs in each of the sequences. NA if absent }
    \item{\code{strand}:}{Object of class \code{"character"} character vector indicating the strand on which the motif is positionned. '+' for direct strand, '-' for reverse complement, NA if motif absent }
    \item{\code{motifList}:}{Object of class \code{"list"} list of the motifs: each element corresponds to a sequence and is the motif (or NA) present in that sequence. }
    \item{\code{objectCall}:}{Object of class \code{"call"} stores how
      the object was created }
    \item{\code{sequences}:}{Object of class \code{"list"} or
      ]code{"matrix"} containing the sequences, or of class
      \code{"character"} for the path to the text file containing the
      sequences in FASTA format}
    \item{\code{positions}:}{Object of class \code{"numeric"} containing
      the starting positions of the motif in the 'sequences', or of
      class \code{"character"} for the path of the text file containing
      the starting positions}
    \item{\code{w}:}{Object of class \code{"numeric"}, integer
      indicating the width of the motif}
    \item{\code{pwm}:}{Object of class \code{"matrix"}, if available,
      the position weight matrix}
    \item{\code{description}:}{Objecxt of class \code{"character"}, a
      brief explanation of the object }
  }
}
\section{Methods}{
\describe{
   \item{summary}{\code{signature(object = "motif")}: returns the position weight matrix}
   \item{plot}{\code{signature(x = "motif")}: plots the information content corresponding to the position weight matrix, anf for each position indicates the corresponding nucleotide from the consensus}  
}
}
\references{ ~put references to the literature/web site here ~ }
\author{
Fabian Gallusser, \email{fgallusser@berkeley.edu
}
}
\note{ 
If the position weight matrix is not given as an argument (default is NULL), it is estimated from the list of motifs.
For the motif() command, if the default NULL value is kept for the 'sequences' and/or 'positions' arguments, vignette(s) allowing to browse and select the necessary file(s) will open (requires the tkWidget library)
}
\seealso{
  \code{\link{seqInfo-class}}, \code{\link{multiMotif-class}}, \code{\link{seqData-class}}
}
\examples{
data(seqs)
data(starts)
motif(sequences=seqs, positions=starts, w=8)
}
\keyword{classes}
