\name{multiSeqGen}
\alias{multiSeqGen}
\alias{multiMotifSeqGen}
\title{ artificial sequence generators}
\description{
  These two functions generate artificial sequences which can be used to
  evaluate motif detection performances. \code{multiSeqGen} will only
  insert at most one motif of a given position weight matrix per
  sequence but allows two different motifs to be inserted according to
  two different position weight matrices, whereas
 \code{multMotifSeqGen} may insert a same motif more than once.
}
\usage{
multiSeqGen(num, markov = NULL, backTransMat, proba1, forTransMat1, proba2 = 0, forTransMat2 = NULL, pfirst = 0.5, pstrand = 0, outType = "o", path = "", filename = "seqFile", dis = rnorm, para1 = 100, para2 = 20)
multiMotifSeqGen(num, markov = NULL, backTransMat, forTransMat, lambda, pstrand = 0, outType = "o", path = "", filename = "seqFile", dis = rnorm, para1 = 100, para2 = 20)
}
\arguments{
  \item{num}{ integer argument, number of sequences to be generated}
  \item{markov}{ markov order. If missing, it will be evaluated from the dimension of the transition matrix}
  \item{backTransMat}{ background transition matrix of dimensions 4\^'markov' by 4 }
  \item{proba1}{ probability of insertion of the first motif in the sequence }
  \item{proba2}{ probability of insertion of the second motif in the sequence, 0 by default }
  \item{forTransMat1}{ position weight matrix of the first motif}
  \item{forTransMat2}{ position weight matrix of the second motif, NULL matrix by default}
  \item{forTransMat} {position weight matrix of the motif in \code{multiMotifSeqGen}}
  \item{pfirst}{ probability that the first motif is inserted before the second }
  \item{pstrand}{ probability of the motif or motifs to be inserted on the direct strand. If two motifs are inserted according to different position weight 
matrices, they are systematically inserted on the same strand.}
  \item{outType}{ the type of output to be created: 'o' or 'O' for R objects only, 't' or 'T' for text files only, 'b' or 'B' for both.}
  \item{path}{ the path to the folder where the text files are to be created}
  \item{filename}{ the name to be given to the generated text files: the
    files will be filename\_seq.txt for the sequences and filename\_pos.txt for information
on the positions and strands where the motifs were inserted}
  \item{dis}{ the distribution of the length od the sequences, normal distribution by default}
  \item{para1}{ first parameter of the distribution, expected value of 100 by default}
  \item{para2}{ second parameter of the distribution, standard deviation of 20 by default}
  \item{lambda}{ rate of the exponential distribution determining the distance between motifs for \code{multiMotifSeqGen}}
}
\details{}
\value{
  If 'outType' is 'o' or 'O':
 only R objects are created, an object of class \code{seqData} if \code{multiSeqGen} was used, of class \code{multiSeqData} if \code{multiMotifSeqGen} was called.
  If 'outType' is 't' or 'T':
 two text files are produced. The first with name ending in \_seq.text contains the sequences in FASTA format, the other ending in \_pos.txt contains the information
related to where the motifs are inserted: nucleotide position and strand 
  If 'outType' is 'b' or 'B':
 both R objects and text files are generated
}
\references{ }
\author{
Fabian Gallusser, \email{fgallusser@berkeley.edu}
}
\note{ }
\seealso{
  \code{\link{seqData-class}}, \code{\link{multiSeqData-class}}
}
\examples{
#Generate 20 sequences according to a ZOOPS model, with only one possible motif inserted with probability 0.8.
#The lengths are distibuted according to a normal distribution with the
#default settings.

data(transmat)
data(motifpwm)
genSeq = multiSeqGen(num=20, markov=NULL, backTransMat=transmat, proba1=0.8, forTransMat1=motifpwm, proba2=0, forTransMat2=NULL, pfirst=0.5, pstrand=0.5, outType="b", path="", filename="genSeq", dis=rnorm, para1=100, para2=20)

#The sequences have both been saved in text format in the working directory, and as a seqData object.
#A seqData object can be plotted, yielding three succesive plots: a
#histogram representing the distribution of the lengths, the information
#content profile and an image of the background.

plot(genSeq)



#Generate 20 sequences with only one possible motif that can be repeated any number of time.
#The lengths are distibuted according to a normal distribution with the
#default settings.

genSeq2 = multiMotifSeqGen(num=20, markov=NULL,
backTransMat=transmat,forTransMat=motifpwm, lambda=0.01, pstrand=0.5,
outType="b", path="", filename="genSeq2", dis=rnorm, para1=100,
para2=20)

#The sequences have both been saved in text format in the working directory, and as a multiSeqData object.
#A multiSeqData object can be plotted, yielding four succesive plots: a histogram representing the distribution of the lengths, the information content profile, a histogram representing the frequency of motifs per sequence and an image of the background.
plot(genSeq2)



}
\keyword{misc}
