#define LLR_RANGE 200           /* range of scaled LLR statistic */

/* probability distribution */ 
typedef struct distr {
  int w;			/* maximum width of alignment */
  double alpha;			/* scale factor for distribution */
  int *range;			/* range of distributions w = [1..w] */
  int *offset;			/* offset of distributions w = [1..w] */
  double **d;			/* array of (log) distributions w = [1..w] */
  double **cdf;			/* array of llr (log) 1 - CDFs w = [1..w] */
  double mean;			/* mean of llr for w = 1 */
} DISTR;


/*
  functions
*/
extern int int_compare(
		       const void *v1,
		       const void *v2
		       );

extern double log_comb(
		       int m,
		       int n
		       ); 


extern void init_llr_pv_tables(
			       int min,				/* minimum number of sites */
			       int max,				/* maximum number of sites */
			       double *back				/* background frequencies */
			       );

double *llr_distr(
		  int A,				/* dimension of discrete distribution */
		  double *dd,				/* discrete distribution */
		  int N,				/* number of samples */
		  int desired_range,			/* desired range for scaled LLR */
		  double frac,				/* fraction of scores to use */
		  double *alpha,			/* scale factor for scaled LLR */
		  int *offset,				/* prob[0] = prob(offset) */
		  int *range				/* range for scaled LLR */
		  );

double *sum_distr(
		  double *d1,				/* (log) distribution of RV1 */
		  int r1,				/* range of RV1 */
		  double *d2,				/* (log) distribution of RV2 */
		  int r2, 				/* range of RV2 */
		  int *r_sum				/* range of sum of RV1 and RV2 */
		  );

static double *cdf(
		   double *d,				/* integer valued distribution */
		   int r	      			/* range [0..r] */
		   );

int get_scaled_llr(
		   double alpha,		/* scale factor */
		   int A,			/* size of alphabet */
		   double *o,			/* observed counts */
		   double *e 			/* expected counts */
		   );


extern double get_llr_pv(
			 double llr,				/* log likelihood ratio */
			 double n,				/* wgtd number sequences in alignment */
			 int w,	             			/* width of alignment */
			 int range,				/* desired range for resolution */
			 double frac,				/* speedup factor */
			 int alength,				/* length of alphabet */
			 double *dd 				/* alphabet frequency distribution */
			 );

extern double get_llr_mean(
			   double n 				/* wgt. number sequences in alignment */
			   );


extern double get_log_nalign(
			     MTYPE mtype,				/* type of model */
			     int w,	 				/* width of motif */
			     int N,					/* number of occurrences */
			     DATASET *dataset  				/* the dataset */
			     );

extern double log_qfast(
			int n,			/* number of random variables in product */
			double logk			/* product of random variables */
			);
